@@ -19,29 +19,29 @@ on_blue(red('hello'))+' '+on_red(blue('there'))+green('!')
 red('hello')
 """
 
+import re
+from cwcwidth import wcswidth, wcwidth
+from itertools import chain
 from typing import (
-    Iterator,
-    Text,
-    Tuple,
-    List,
-    Union,
-    Optional,
     Any,
-    Mapping,
-    cast,
-    MutableMapping,
-    no_type_check,
-    Type,
-    cast,
     Callable,
+    Dict,
     Iterable,
+    Iterator,
+    List,
+    Mapping,
+    MutableMapping,
+    Optional,
+    Tuple,
+    Union,
+    cast,
+    no_type_check,
 )
 
-
-import itertools
-import re
-import sys
-from cwcwidth import wcswidth, wcwidth
+try:
+    from functools import cached_property
+except ImportError:
+    from backports.cached_property import cached_property  # type: ignore
 
 from .escseqparse import parse, remove_ansi
 from .termformatconstants import (
@@ -56,29 +56,27 @@ from .termformatconstants import (
     seq,
 )
 
-one_arg_xforms = {
+one_arg_xforms: Mapping[str, Callable[[str], str]] = {
     "bold": lambda s: seq(STYLES["bold"]) + s + seq(RESET_ALL),
     "dark": lambda s: seq(STYLES["dark"]) + s + seq(RESET_ALL),
     "underline": lambda s: seq(STYLES["underline"]) + s + seq(RESET_ALL),
     "blink": lambda s: seq(STYLES["blink"]) + s + seq(RESET_ALL),
     "invert": lambda s: seq(STYLES["invert"]) + s + seq(RESET_ALL),
-}  # type: Mapping[Text, Callable[[Text], Text]]
+}
 
-two_arg_xforms = {
+two_arg_xforms: Mapping[str, Callable[[str, int], str]] = {
     "fg": lambda s, v: "{}{}{}".format(seq(v), s, seq(RESET_FG)),
     "bg": lambda s, v: seq(v) + s + seq(RESET_BG),
-}  # type: Mapping[Text, Callable[[Text, int], Text]]
+}
 
 # TODO unused, remove this in next major release
-xforms = (
-    {}
-)  # type: MutableMapping[Text, Union[Callable[[Text], Text], Callable[[Text, int], Text]]]
+xforms: MutableMapping[str, Union[Callable[[str], str], Callable[[str, int], str]]] = {}
 xforms.update(one_arg_xforms)
 xforms.update(two_arg_xforms)
 
 
-class FrozenDict(dict):
-    """Immutable dictionary class"""
+class FrozenAttributes(Dict[str, Union[int, bool]]):
+    """Immutable dictionary class for format string attributes"""
 
     @no_type_check
     def __setitem__(self, key, value):
@@ -88,17 +86,14 @@ class FrozenDict(dict):
     def update(self, *args, **kwds):
         raise Exception("Cannot change value.")
 
-    def extend(self, dictlike):
-        # type: (Mapping[str, Union[int, bool]]) -> FrozenDict
-        return FrozenDict(itertools.chain(self.items(), dictlike.items()))
+    def extend(self, dictlike: Mapping[str, Union[int, bool]]) -> "FrozenAttributes":
+        return FrozenAttributes(chain(self.items(), dictlike.items()))
 
-    def remove(self, *keys):
-        # type: (*str) -> FrozenDict
-        return FrozenDict((k, v) for k, v in self.items() if k not in keys)
+    def remove(self, *keys: str) -> "FrozenAttributes":
+        return FrozenAttributes((k, v) for k, v in self.items() if k not in keys)
 
 
-def stable_format_dict(d):
-    # type: (Mapping) -> str
+def stable_format_dict(d: Mapping) -> str:
     """A sorted, python2/3 stable formatting of a dictionary.
 
     Does not work for dicts with unicode strings as values."""
@@ -119,45 +114,38 @@ class Chunk:
 
     Subject to change, not part of the API"""
 
-    def __init__(self, string, atts=None):
-        # type: (Text, Mapping[str, Union[int, bool]]) -> None
-        if atts is None:
-            atts = {}
+    def __init__(
+        self, string: str, atts: Optional[Mapping[str, Union[int, bool]]] = None
+    ):
         if not isinstance(string, str):
             raise ValueError("unicode string required, got %r" % string)
-        self._s = string  # type: Text
-        self._atts = FrozenDict(atts)
+        self._s = string
+        self._atts = FrozenAttributes(atts if atts else {})
 
     @property
-    def s(self):
-        # type: () -> Text
+    def s(self) -> str:
         return self._s
 
     @property
-    def atts(self):
-        # type: () -> Mapping[str, Union[int, bool]]
+    def atts(self) -> FrozenAttributes:
         "Attributes, e.g. {'fg': 34, 'bold': True} where 34 is the escape code for ..."
         return self._atts
 
-    def __len__(self):
-        # type: () -> int
+    def __len__(self) -> int:
         return len(self._s)
 
     @property
-    def width(self):
-        # type: () -> int
+    def width(self) -> int:
         width = wcswidth(self._s)
         if len(self._s) > 0 and width < 1:
             raise ValueError("Can't calculate width of string %r" % self._s)
         return width
 
-    # TODO cache this
-    @property
-    def color_str(self):
-        # type: () -> Text
+    @cached_property
+    def color_str(self) -> str:
         "Return an escape-coded string to write to the terminal."
-        s = self.s
-        for k, v in sorted(self.atts.items()):
+        s = self._s
+        for k, v in sorted(self._atts.items()):
             # (self.atts sorted for the sake of always acting the same.)
             if k not in xforms:
                 # Unsupported SGR code
@@ -167,40 +155,35 @@ class Chunk:
             elif v is True:
                 s = one_arg_xforms[k](s)
             else:
+                # TODO: What's the purpose of this code? It will never be executed.
                 s = two_arg_xforms[k](s, v)
         return s
 
-    def __str__(self):
-        # type: () -> Text
+    def __str__(self) -> str:
         value = self.color_str
         if isinstance(value, bytes):
             return value.decode("utf8", "replace")
         return value
 
-    def __eq__(self, other):
-        # type: (Chunk, Any) -> bool
+    def __eq__(self, other: Any) -> bool:
         if not isinstance(other, Chunk):
-            return False
-        return self.s == other.s and self.atts == other.atts
+            return NotImplemented
+        return self._s == other._s and self._atts == other._atts
 
-    def __hash__(self):
-        # type: () -> int
-        return hash((self.s, self.atts))
+    def __hash__(self) -> int:
+        return hash((self._s, self._atts))
 
-    def __repr__(self):
-        # type: () -> str
+    def __repr__(self) -> str:
         return "Chunk({s}{sep}{atts})".format(
-            s=repr(self.s),
-            sep=", " if self.atts else "",
-            atts=stable_format_dict(self.atts) if self.atts else "",
+            s=repr(self._s),
+            sep=", " if self._atts else "",
+            atts=stable_format_dict(self._atts) if self._atts else "",
         )
 
-    def repr_part(self):
-        # type: () -> str
+    def repr_part(self) -> str:
         """FmtStr repr is build by concatenating these."""
 
-        def pp_att(att):
-            # type: (str) -> str
+        def pp_att(att: str) -> str:
             if att == "fg":
                 return FG_NUMBER_TO_COLOR[self.atts[att]]
             elif att == "bg":
@@ -208,15 +191,14 @@ class Chunk:
             else:
                 return att
 
-        atts_out = {k: v for (k, v) in self.atts.items() if v}
+        atts_out = {k: v for (k, v) in self._atts.items() if v}
         return (
             "".join(pp_att(att) + "(" for att in sorted(atts_out))
-            + repr(self.s)
+            + repr(self._s)
             + ")" * len(atts_out)
         )
 
-    def splitter(self):
-        # type: () -> ChunkSplitter
+    def splitter(self) -> "ChunkSplitter":
         """
         Returns a view of this Chunk from which new Chunks can be requested.
         """
@@ -228,8 +210,7 @@ class ChunkSplitter:
     View of a Chunk for breaking it into smaller Chunks.
     """
 
-    def __init__(self, chunk):
-        # type: (Chunk) -> None
+    def __init__(self, chunk: Chunk) -> None:
         self.chunk = chunk
         self.internal_offset = 0  # index into chunk.s
         self.internal_width = 0  # width of chunks.s[:self.internal_offset]
@@ -238,14 +219,12 @@ class ChunkSplitter:
             divides.append(divides[-1] + wcwidth(c))
         self.divides = divides
 
-    def reinit(self, chunk):
-        # type: (Chunk) -> None
+    def reinit(self, chunk: Chunk) -> None:
         """Reuse an existing Splitter instance for speed."""
         # TODO benchmark to prove this is worthwhile
         self.__init__(chunk)  # type: ignore
 
-    def request(self, max_width):
-        # type: (int) -> Optional[Tuple[int, Chunk]]
+    def request(self, max_width: int) -> Optional[Tuple[int, Chunk]]:
         """Requests a sub-chunk of max_width or shorter. Returns None if no chunks left."""
         if max_width < 1:
             raise ValueError("requires positive integer max_width")
@@ -261,7 +240,7 @@ class ChunkSplitter:
         replacement_char = " "
 
         while True:
-            w = wcswidth(s[i])
+            w = wcswidth(s[i], None)
 
             # If adding a character puts us over the requested width, return what we've got so far
             if width + w > max_width:
@@ -305,22 +284,20 @@ class ChunkSplitter:
 class FmtStr:
     """A string whose substrings carry attributes."""
 
-    def __init__(self, *components):
-        # type: (*Chunk) -> None
+    def __init__(self, *components: Chunk) -> None:
         # These assertions below could be useful for debugging, but slow things down considerably
         # assert all([len(x) > 0 for x in components])
         # self.chunks = [x for x in components if len(x) > 0]
         self.chunks = list(components)
 
         # caching these leads to a significant speedup
-        self._unicode = None  # type: Optional[Text]
-        self._len = None  # type: Optional[int]
-        self._s = None  # type: Optional[Text]
-        self._width = None  # type: Optional[int]
+        self._unicode: Optional[str] = None
+        self._len: Optional[int] = None
+        self._s: Optional[str] = None
+        self._width: Optional[int] = None
 
     @classmethod
-    def from_str(cls, s):
-        # type: (Union[Text]) -> FmtStr
+    def from_str(cls, s: str) -> "FmtStr":
         r"""
         Return a FmtStr representing input.
 
@@ -350,13 +327,12 @@ class FmtStr:
                         )
                         chunks.append(Chunk(x, atts=atts))
                     else:
-                        raise Exception("logic error")
+                        raise TypeError(f"Expected dict or str, not {type(x)}")
                 return FmtStr(*chunks)
         else:
             return FmtStr(Chunk(s))
 
-    def copy_with_new_str(self, new_str):
-        # type: (Text) -> FmtStr
+    def copy_with_new_str(self, new_str: str) -> "FmtStr":
         """Copies the current FmtStr's attributes while changing its string."""
         # What to do when there are multiple Chunks with conflicting atts?
         old_atts = {
@@ -364,13 +340,13 @@ class FmtStr:
         }
         return FmtStr(Chunk(new_str, old_atts))
 
-    def setitem(self, startindex, fs):
-        # type: (int, Union[Text, FmtStr]) -> FmtStr
+    def setitem(self, startindex: int, fs: Union[str, "FmtStr"]) -> "FmtStr":
         """Shim for easily converting old __setitem__ calls"""
         return self.setslice_with_length(startindex, startindex + 1, fs, len(self))
 
-    def setslice_with_length(self, startindex, endindex, fs, length):
-        # type: (int, int, Union[Text, FmtStr], int) -> FmtStr
+    def setslice_with_length(
+        self, startindex: int, endindex: int, fs: Union[str, "FmtStr"], length: int
+    ) -> "FmtStr":
         """Shim for easily converting old __setitem__ calls"""
         if len(self) < startindex:
             fs = " " * (startindex - len(self)) + fs
@@ -384,8 +360,9 @@ class FmtStr:
             )
         return result
 
-    def splice(self, new_str, start, end=None):
-        # type: (Union[Text, FmtStr], int, int) -> FmtStr
+    def splice(
+        self, new_str: Union[str, "FmtStr"], start: int, end: Optional[int] = None
+    ) -> "FmtStr":
         """Returns a new FmtStr with the input string spliced into the
         the original FmtStr at start and end.
         If end is provided, new_str will replace the substring self.s[start:end-1].
@@ -431,25 +408,23 @@ class FmtStr:
             new_components.extend(new_fs.chunks)
             inserted = True
 
-        return FmtStr(*[s for s in new_components if s.s])
+        return FmtStr(*(s for s in new_components if s.s))
 
-    def append(self, string):
-        # type: (Union[Text, FmtStr]) -> FmtStr
+    def append(self, string: Union[str, "FmtStr"]) -> "FmtStr":
         return self.splice(string, len(self.s))
 
-    def copy_with_new_atts(self, **attributes):
-        # type: (**Union[bool, int]) -> FmtStr
+    def copy_with_new_atts(self, **attributes: Union[bool, int]) -> "FmtStr":
         """Returns a new FmtStr with the same content but new formatting"""
 
-        result = FmtStr(*[Chunk(bfs.s, bfs.atts.extend(attributes)) for bfs in self.chunks])  # type: ignore
-        return result
+        return FmtStr(
+            *(Chunk(bfs.s, bfs.atts.extend(attributes)) for bfs in self.chunks)
+        )
 
-    def join(self, iterable):
-        # type: (Iterable[Union[Text, FmtStr]]) -> FmtStr
+    def join(self, iterable: Iterable[Union[str, "FmtStr"]]) -> "FmtStr":
         """Joins an iterable yielding strings or FmtStrs with self as separator"""
-        before = []  # type: List[Chunk]
-        chunks = []  # type: List[Chunk]
-        for i, s in enumerate(iterable):
+        before: List[Chunk] = []
+        chunks: List[Chunk] = []
+        for s in iterable:
             chunks.extend(before)
             before = self.chunks
             if isinstance(s, FmtStr):
@@ -461,8 +436,12 @@ class FmtStr:
         return FmtStr(*chunks)
 
     # TODO make this split work like str.split
-    def split(self, sep=None, maxsplit=None, regex=False):
-        # type: (Text, int, bool) -> List[FmtStr]
+    def split(
+        self,
+        sep: Optional[str] = None,
+        maxsplit: Optional[int] = None,
+        regex: bool = False,
+    ) -> List["FmtStr"]:
         """Split based on separator, optionally using a regex.
 
         Capture groups are ignored in regex, the whole pattern is matched
@@ -478,13 +457,12 @@ class FmtStr:
         return [
             self[start:end]
             for start, end in zip(
-                [0] + [m.end() for m in matches],
-                [m.start() for m in matches] + [len(s)],
+                chain((0,), (m.end() for m in matches)),
+                chain((m.start() for m in matches), (len(s),)),
             )
         ]
 
-    def splitlines(self, keepends=False):
-        # type: (bool) -> List[FmtStr]
+    def splitlines(self, keepends: bool = False) -> List["FmtStr"]:
         """Return a list of lines, split on newline characters,
         include line boundaries, if keepends is true."""
         lines = self.split("\n")
@@ -496,8 +474,7 @@ class FmtStr:
 
     # proxying to the string via __getattr__ is insufficient
     # because we shouldn't drop foreground or formatting info
-    def ljust(self, width, fillchar=None):
-        # type: (int, Text) -> FmtStr
+    def ljust(self, width: int, fillchar: Optional[str] = None) -> "FmtStr":
         """S.ljust(width[, fillchar]) -> string
 
         If a fillchar is provided, less formatting information will be preserved
@@ -512,8 +489,7 @@ class FmtStr:
             uniform = self.new_with_atts_removed("bg")
             return uniform + fmtstr(to_add, **self.shared_atts) if to_add else uniform
 
-    def rjust(self, width, fillchar=None):
-        # type: (int, Text) -> FmtStr
+    def rjust(self, width: int, fillchar: Optional[str] = None) -> "FmtStr":
         """S.rjust(width[, fillchar]) -> string
 
         If a fillchar is provided, less formatting information will be preserved
@@ -528,15 +504,13 @@ class FmtStr:
             uniform = self.new_with_atts_removed("bg")
             return fmtstr(to_add, **self.shared_atts) + uniform if to_add else uniform
 
-    def __str__(self):
-        # type: () -> Text
+    def __str__(self) -> str:
         if self._unicode is not None:
             return self._unicode
         self._unicode = "".join(str(fs) for fs in self.chunks)
         return self._unicode
 
-    def __len__(self):
-        # type: () -> int
+    def __len__(self) -> int:
         if self._len is not None:
             return self._len
         value = sum(len(fs) for fs in self.chunks)
@@ -544,8 +518,7 @@ class FmtStr:
         return value
 
     @property
-    def width(self):
-        # type: () -> int
+    def width(self) -> int:
         """The number of columns it would take to display this string."""
         if self._width is not None:
             return self._width
@@ -553,57 +526,50 @@ class FmtStr:
         self._width = value
         return value
 
-    def width_at_offset(self, n):
-        # type: (int) -> int
+    def width_at_offset(self, n: int) -> int:
         """Returns the horizontal position of character n of the string"""
         # TODO make more efficient?
         width = wcswidth(self.s, n)
         assert width != -1
         return width
 
-    def __repr__(self):
-        # type: () -> str
+    def __repr__(self) -> str:
         return "+".join(fs.repr_part() for fs in self.chunks)
 
-    def __eq__(self, other):
-        # type: (Any) -> bool
+    def __eq__(self, other: Any) -> bool:
         if isinstance(other, (str, bytes, FmtStr)):
             return str(self) == str(other)
-        return False
+        return NotImplemented
 
-    def __hash__(self):
-        # type: () -> int
+    def __hash__(self) -> int:
         return hash(str(self))
 
-    def __add__(self, other):
-        # type: (Union[FmtStr, Text]) -> FmtStr
+    def __add__(self, other: Union["FmtStr", str]) -> "FmtStr":
         if isinstance(other, FmtStr):
             return FmtStr(*(self.chunks + other.chunks))
         elif isinstance(other, (bytes, str)):
             return FmtStr(*(self.chunks + [Chunk(other)]))
-        else:
-            raise TypeError(f"Can't add {self!r} and {other!r}")
 
-    def __radd__(self, other):
-        # type: (Union[FmtStr, Text]) -> FmtStr
+        return NotImplemented
+
+    def __radd__(self, other: Union["FmtStr", str]) -> "FmtStr":
         if isinstance(other, FmtStr):
             return FmtStr(*(x for x in (other.chunks + self.chunks)))
         elif isinstance(other, (bytes, str)):
             return FmtStr(*(x for x in ([Chunk(other)] + self.chunks)))
-        else:
-            raise TypeError("Can't add those")
 
-    def __mul__(self, other):
-        # type: (int) -> FmtStr
+        return NotImplemented
+
+    def __mul__(self, other: int) -> "FmtStr":
         if isinstance(other, int):
-            return sum([self for _ in range(other)], FmtStr())
-        raise TypeError("Can't multiply those")
+            return sum((self for _ in range(other)), FmtStr())
+
+        return NotImplemented
 
     # TODO ensure empty FmtStr isn't a problem
 
     @property
-    def shared_atts(self):
-        # type: () -> Mapping[str, Union[int, bool]]
+    def shared_atts(self) -> Mapping[str, Union[int, bool]]:
         """Gets atts shared among all nonzero length component Chunks"""
         # TODO cache this, could get ugly for large FmtStrs
         atts = {}
@@ -618,11 +584,10 @@ class FmtStr:
                 atts[att] = first.atts[att]
         return atts
 
-    def new_with_atts_removed(self, *attributes):
-        # type: (*Text) -> FmtStr
+    def new_with_atts_removed(self, *attributes: str) -> "FmtStr":
         """Returns a new FmtStr with the same content but some attributes removed"""
 
-        result = FmtStr(*[Chunk(bfs.s, bfs.atts.remove(*attributes)) for bfs in self.chunks])  # type: ignore
+        result = FmtStr(*(Chunk(bfs.s, bfs.atts.remove(*attributes)) for bfs in self.chunks))  # type: ignore
         return result
 
     @no_type_check
@@ -644,8 +609,7 @@ class FmtStr:
         return func_help
 
     @property
-    def divides(self):
-        # type: () -> List[int]
+    def divides(self) -> List[int]:
         """List of indices of divisions between the constituent chunks."""
         acc = [0]
         for s in self.chunks:
@@ -653,15 +617,13 @@ class FmtStr:
         return acc
 
     @property
-    def s(self):
-        # type: () -> Text
+    def s(self) -> str:
         if self._s is not None:
             return self._s
         self._s = "".join(fs.s for fs in self.chunks)
         return self._s
 
-    def __getitem__(self, index):
-        # type: (Union[int, slice]) -> FmtStr
+    def __getitem__(self, index: Union[int, slice]) -> "FmtStr":
         index = normalize_slice(len(self), index)
         counter = 0
         parts = []
@@ -681,10 +643,9 @@ class FmtStr:
                 break
         return FmtStr(*parts) if parts else fmtstr("")
 
-    def width_aware_slice(self, index):
-        # type: (Union[int, slice]) -> FmtStr
+    def width_aware_slice(self, index: Union[int, slice]) -> "FmtStr":
         """Slice based on the number of columns it would take to display the substring."""
-        if wcswidth(self.s) == -1:
+        if wcswidth(self.s, None) == -1:
             raise ValueError("bad values for width aware slicing")
         index = normalize_slice(self.width, index)
         counter = 0
@@ -705,20 +666,18 @@ class FmtStr:
                 break
         return FmtStr(*parts) if parts else fmtstr("")
 
-    def width_aware_splitlines(self, columns):
-        # type: (int) -> Iterator[FmtStr]
+    def width_aware_splitlines(self, columns: int) -> Iterator["FmtStr"]:
         """Split into lines, pushing doublewidth characters at the end of a line to the next line.
 
         When a double-width character is pushed to the next line, a space is added to pad out the line.
         """
         if columns < 2:
             raise ValueError("Column width %s is too narrow." % columns)
-        if wcswidth(self.s) == -1:
+        if wcswidth(self.s, None) == -1:
             raise ValueError("bad values for width aware slicing")
         return self._width_aware_splitlines(columns)
 
-    def _width_aware_splitlines(self, columns):
-        # type: (int) -> Iterator[FmtStr]
+    def _width_aware_splitlines(self, columns: int) -> Iterator["FmtStr"]:
         splitter = self.chunks[0].splitter()
         chunks_of_line = []
         width_of_line = 0
@@ -740,8 +699,7 @@ class FmtStr:
         if chunks_of_line:
             yield FmtStr(*chunks_of_line)
 
-    def _getitem_normalized(self, index):
-        # type: (Union[int, slice]) -> FmtStr
+    def _getitem_normalized(self, index: Union[int, slice]) -> "FmtStr":
         """Builds the more compact fmtstrs by using fromstr( of the control sequences)"""
         index = normalize_slice(len(self), index)
         counter = 0
@@ -756,17 +714,14 @@ class FmtStr:
                 break
         return fmtstr(output)
 
-    def __setitem__(self, index, value):
-        # type: (int, Any) -> None
+    def __setitem__(self, index: int, value: Any) -> None:
         raise Exception("No!")
 
-    def copy(self):
-        # type: () -> FmtStr
+    def copy(self) -> "FmtStr":
         return FmtStr(*self.chunks)
 
 
-def interval_overlap(a, b, x, y):
-    # type: (int, int, int, int) -> int
+def interval_overlap(a: int, b: int, x: int, y: int) -> int:
     """Returns by how much two intervals overlap
 
     assumed that a <= b and x <= y"""
@@ -782,8 +737,7 @@ def interval_overlap(a, b, x, y):
         assert False
 
 
-def width_aware_slice(s, start, end, replacement_char=" "):
-    # type: (Text, int, int, Text) -> Text
+def width_aware_slice(s: str, start: int, end: int, replacement_char: str = " ") -> str:
     """
     >>> width_aware_slice(u'a\uff25iou', 0, 2)[1] == u' '
     True
@@ -807,8 +761,7 @@ def width_aware_slice(s, start, end, replacement_char=" "):
     return "".join(new_chunk_chars)
 
 
-def linesplit(string, columns):
-    # type: (Union[Text, FmtStr], int) -> List[FmtStr]
+def linesplit(string: Union[str, FmtStr], columns: int) -> List[FmtStr]:
     """Returns a list of lines, split on the last possible space of each line.
 
     Split spaces will be removed. Whitespaces will be normalized to one space.
@@ -853,8 +806,7 @@ def linesplit(string, columns):
     return lines
 
 
-def normalize_slice(length, index):
-    # type: (int, Union[int, slice]) -> slice
+def normalize_slice(length: int, index: Union[int, slice]) -> slice:
     "Fill in the Nones in a slice."
     is_int = False
     if isinstance(index, int):
@@ -876,16 +828,17 @@ def normalize_slice(length, index):
     return index
 
 
-def parse_args(args, kwargs):
-    # type: (Tuple[Union[bytes, str], ...], MutableMapping[str, Union[int, bool, str]]) -> Mapping[str, Union[int, bool]]
+def parse_args(
+    args: Tuple[str, ...],
+    kwargs: MutableMapping[str, Union[int, bool, str]],
+) -> Mapping[str, Union[int, bool]]:
     """Returns a kwargs dictionary by turning args into kwargs"""
     if "style" in kwargs:
         args += (cast(str, kwargs["style"]),)
         del kwargs["style"]
     for arg in args:
-        arg = cast(str, arg)
-        if not isinstance(arg, (bytes, str)):
-            raise ValueError("args must be strings:" + repr(args))
+        if not isinstance(arg, str):
+            raise ValueError(f"args must be strings: {arg!r}")
         if arg.lower() in FG_COLORS:
             if "fg" in kwargs:
                 raise ValueError("fg specified twice")
@@ -897,25 +850,24 @@ def parse_args(args, kwargs):
         elif arg.lower() in STYLES:
             kwargs[arg] = True
         else:
-            raise ValueError("couldn't process arg: " + repr(arg))
+            raise ValueError(f"couldn't process arg: {args!r}")
     for k in kwargs:
-        if k not in ["fg", "bg"] + list(STYLES.keys()):
+        if k not in ("fg", "bg") and k not in STYLES.keys():
             raise ValueError("Can't apply that transformation")
     if "fg" in kwargs:
         if kwargs["fg"] in FG_COLORS:
             kwargs["fg"] = FG_COLORS[cast(str, kwargs["fg"])]
         if kwargs["fg"] not in list(FG_COLORS.values()):
-            raise ValueError("Bad fg value: %r" % kwargs["fg"])
+            raise ValueError(f"Bad fg value: {kwargs['fg']!r}")
     if "bg" in kwargs:
         if kwargs["bg"] in BG_COLORS:
             kwargs["bg"] = BG_COLORS[cast(str, kwargs["bg"])]
         if kwargs["bg"] not in list(BG_COLORS.values()):
-            raise ValueError("Bad bg value: %r" % kwargs["bg"])
+            raise ValueError(f"Bad bg value: {kwargs['bg']!r}")
     return cast(MutableMapping[str, Union[int, bool]], kwargs)
 
 
-def fmtstr(string, *args, **kwargs):
-    # type: (Union[Text, FmtStr], *Any, **Any) -> FmtStr
+def fmtstr(string: Union[str, FmtStr], *args: Any, **kwargs: Any) -> FmtStr:
     """
     Convenience function for creating a FmtStr
 
@@ -925,14 +877,10 @@ def fmtstr(string, *args, **kwargs):
     on_red(bold(blue('blarg')))
     """
     atts = parse_args(args, kwargs)
-    if isinstance(string, FmtStr):
-        pass
-    elif isinstance(string, (bytes, str)):
+    if isinstance(string, str):
         string = FmtStr.from_str(string)
-    else:
+    elif not isinstance(string, FmtStr):
         raise ValueError(
-            "Bad Args: {!r} (of type {}), {!r}, {!r}".format(
-                string, type(string), args, kwargs
-            )
+            f"Bad Args: {string!r} (of type {type(string)}), {args!r}, {kwargs!r}"
         )
     return string.copy_with_new_atts(**atts)
