@@ -1,20 +1,17 @@
 # All windows write only unicode to the terminal -
-# that's what blessings does, so we match it.
+# that's what blessed does, so we match it.
 
-from __future__ import unicode_literals
 
 from typing import (
+    ContextManager,
     Optional,
-    Text,
     IO,
     Dict,
-    Generic,
+    Sequence,
     TypeVar,
     Type,
     Tuple,
     Callable,
-    Any,
-    ByteString,
     cast,
     TextIO,
     Union,
@@ -22,12 +19,11 @@ from typing import (
 )
 from types import TracebackType
 
-import locale
 import logging
 import re
 import sys
 
-import blessings
+import blessed
 
 from .formatstring import fmtstr, FmtStr
 from .formatstringarray import FSArray
@@ -35,90 +31,84 @@ from .termhelpers import Cbreak
 
 logger = logging.getLogger(__name__)
 
-SCROLL_DOWN = "\x1bD"
-FIRST_COLUMN = "\x1b[1G"
-
 
 T = TypeVar("T", bound="BaseWindow")
 
 
-class BaseWindow(object):
-    def __init__(self, out_stream=None, hide_cursor=True):
-        # type: (IO, bool) -> None
+class BaseWindow(ContextManager):
+    def __init__(
+        self, out_stream: Optional[IO] = None, hide_cursor: bool = True
+    ) -> None:
         logger.debug("-------initializing Window object %r------" % self)
         if out_stream is None:
             out_stream = sys.__stdout__
-        self.t = blessings.Terminal(stream=out_stream, force_styling=True)
+        self.t = blessed.Terminal(stream=out_stream, force_styling=True)
         self.out_stream = out_stream
         self.hide_cursor = hide_cursor
-        self._last_lines_by_row = {}  # type: Dict[int, Optional[FmtStr]]
-        self._last_rendered_width = None  # type: Optional[int]
-        self._last_rendered_height = None  # type: Optional[int]
+        self._last_lines_by_row: Dict[int, Optional[FmtStr]] = {}
+        self._last_rendered_width: Optional[int] = None
+        self._last_rendered_height: Optional[int] = None
 
-    def scroll_down(self):
-        # type: () -> None
+    def scroll_down(self) -> None:
         logger.debug("sending scroll down message w/ cursor on bottom line")
 
         # since scroll-down only moves the screen if cursor is at bottom
         with self.t.location(x=0, y=1000000):
-            self.write(SCROLL_DOWN)  # TODO will blessings do this?
+            self.write(self.t.move_down)
 
-    def write(self, msg):
-        # type: (Text) -> None
+    def write(self, msg: str) -> None:
         self.out_stream.write(msg)
         self.out_stream.flush()
 
-    def __enter__(self):
-        # type: (T) -> T
+    def __enter__(self: T) -> T:
         logger.debug("running BaseWindow.__enter__")
         if self.hide_cursor:
             self.write(self.t.hide_cursor)
         return self
 
-    def __exit__(self, type, value, traceback):
-        # type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
+    def __exit__(
+        self,
+        type: Optional[Type[BaseException]] = None,
+        value: Optional[BaseException] = None,
+        traceback: Optional[TracebackType] = None,
+    ) -> None:
         logger.debug("running BaseWindow.__exit__")
         if self.hide_cursor:
             self.write(self.t.normal_cursor)
 
-    def on_terminal_size_change(self, height, width):
-        # type: (int, int) -> None
+    def on_terminal_size_change(self, height: int, width: int) -> None:
         # Changing the terminal size breaks the cache, because it
         # is unknown how the window size change affected scrolling / the cursor
         self._last_lines_by_row = {}
         self._last_rendered_width = width
         self._last_rendered_height = height
 
-    def render_to_terminal(self, array, cursor_pos=(0, 0)):
-        # type: (Union[FSArray, List[FmtStr]], Tuple[int, int]) -> Optional[int]
+    def render_to_terminal(
+        self, array: Union[FSArray, List[FmtStr]], cursor_pos: Tuple[int, int] = (0, 0)
+    ) -> Optional[int]:
         raise NotImplementedError
 
-    def get_term_hw(self):
-        # type: () -> Tuple[int, int]
+    def get_term_hw(self) -> Tuple[int, int]:
         """Returns current terminal height and width"""
         return self.t.height, self.t.width
 
     @property
-    def width(self):
-        # type: () -> int
+    def width(self) -> int:
         "The current width of the terminal window"
         return self.t.width
 
     @property
-    def height(self):
-        # type: () -> int
+    def height(self) -> int:
         "The current width of the terminal window"
         return self.t.height
 
-    def array_from_text(self, msg):
-        # type: (Text) -> FSArray
+    def array_from_text(self, msg: str) -> FSArray:
         """Returns a FSArray of the size of the window containing msg"""
         rows, columns = self.t.height, self.t.width
         return self.array_from_text_rc(msg, rows, columns)
 
     @classmethod
-    def array_from_text_rc(cls, msg, rows, columns):
-        # type: (Text, int, int) -> FSArray
+    def array_from_text_rc(cls, msg: str, rows: int, columns: int) -> FSArray:
         arr = FSArray(0, columns)
         i = 0
         for c in msg:
@@ -131,29 +121,15 @@ class BaseWindow(object):
             i += 1
         return arr
 
-    def fmtstr_to_stdout_xform(self):
-        # type: () -> Callable[[FmtStr], Text]
-        if sys.version_info[0] == 2:
-            if hasattr(self.out_stream, "encoding"):
-                encoding = self.out_stream.encoding
-            else:
-                encoding = locale.getpreferredencoding()
-
-            def for_stdout(s):
-                # type: (FmtStr) -> Text
-                return unicode(s).encode(encoding, "replace")
-
-        else:
-
-            def for_stdout(s):
-                # type: (FmtStr) -> Text
-                return str(s)
+    def fmtstr_to_stdout_xform(self) -> Callable[[FmtStr], str]:
+        def for_stdout(s: FmtStr) -> str:
+            return str(s)
 
         return for_stdout
 
 
-class FullscreenWindow(BaseWindow):
-    """2D-text rendering window that dissappears when its context is left
+class FullscreenWindow(BaseWindow, ContextManager["FullscreenWindow"]):
+    """2D-text rendering window that disappears when its context is left
 
     FullscreenWindow will only render arrays the size of the terminal
     or smaller, and leaves no trace on exit (like top or vim). It never
@@ -168,29 +144,34 @@ class FullscreenWindow(BaseWindow):
         its out_stream; cached writes will be inaccurate.
     """
 
-    def __init__(self, out_stream=None, hide_cursor=True):
-        # type: (IO, bool) -> None
+    def __init__(
+        self, out_stream: Optional[IO] = None, hide_cursor: bool = True
+    ) -> None:
         """Constructs a FullscreenWindow
 
         Args:
             out_stream (file): Defaults to sys.__stdout__
             hide_cursor (bool): Hides cursor while in context
         """
-        BaseWindow.__init__(self, out_stream=out_stream, hide_cursor=hide_cursor)
+        super().__init__(out_stream=out_stream, hide_cursor=hide_cursor)
         self.fullscreen_ctx = self.t.fullscreen()
 
-    def __enter__(self):
-        # type: () -> FullscreenWindow
+    def __enter__(self) -> "FullscreenWindow":
         self.fullscreen_ctx.__enter__()
-        return BaseWindow.__enter__(self)
+        return super().__enter__()
 
-    def __exit__(self, type, value, traceback):
-        # type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
+    def __exit__(
+        self,
+        type: Optional[Type[BaseException]] = None,
+        value: Optional[BaseException] = None,
+        traceback: Optional[TracebackType] = None,
+    ) -> None:
         self.fullscreen_ctx.__exit__(type, value, traceback)
-        BaseWindow.__exit__(self, type, value, traceback)
+        super().__exit__(type, value, traceback)
 
-    def render_to_terminal(self, array, cursor_pos=(0, 0)):
-        # type: (Union[FSArray, List[FmtStr]], Tuple[int, int]) -> None
+    def render_to_terminal(
+        self, array: Union[FSArray, List[FmtStr]], cursor_pos: Tuple[int, int] = (0, 0)
+    ) -> None:
         """Renders array to terminal and places (0-indexed) cursor
 
         Args:
@@ -216,13 +197,10 @@ class FullscreenWindow(BaseWindow):
         if height != self._last_rendered_height or width != self._last_rendered_width:
             self.on_terminal_size_change(height, width)
 
-        current_lines_by_row = {}  # type: Dict[int, Optional[FmtStr]]
-        rows = list(range(height))
-        rows_for_use = rows[: len(array)]
-        rest_of_rows = rows[len(array) :]
+        current_lines_by_row: Dict[int, Optional[FmtStr]] = {}
 
         # rows which we have content for and don't require scrolling
-        for row, line in zip(rows_for_use, array):
+        for row, line in enumerate(array):
             current_lines_by_row[row] = line
             if line == self._last_lines_by_row.get(row, None):
                 continue
@@ -232,7 +210,7 @@ class FullscreenWindow(BaseWindow):
                 self.write(self.t.clear_eol)
 
         # rows onscreen that we don't have content for
-        for row in rest_of_rows:
+        for row in range(len(array), height):
             if self._last_lines_by_row and row not in self._last_lines_by_row:
                 continue
             self.write(self.t.move(row, 0))
@@ -248,7 +226,7 @@ class FullscreenWindow(BaseWindow):
             self.write(self.t.normal_cursor)
 
 
-class CursorAwareWindow(BaseWindow):
+class CursorAwareWindow(BaseWindow, ContextManager["CursorAwareWindow"]):
     """
     Renders to the normal terminal screen and
     can find the location of the cursor.
@@ -265,13 +243,12 @@ class CursorAwareWindow(BaseWindow):
 
     def __init__(
         self,
-        out_stream=None,
-        in_stream=None,
-        keep_last_line=False,
-        hide_cursor=True,
-        extra_bytes_callback=None,
+        out_stream: Optional[IO] = None,
+        in_stream: Optional[IO] = None,
+        keep_last_line: bool = False,
+        hide_cursor: bool = True,
+        extra_bytes_callback: Optional[Callable[[bytes], None]] = None,
     ):
-        # type: (IO, IO, bool, bool, Callable[[ByteString], None]) -> None
         """Constructs a CursorAwareWindow
 
         Args:
@@ -281,17 +258,23 @@ class CursorAwareWindow(BaseWindow):
                 on leaving context
             hide_cursor (bool): Hides cursor while in context
             extra_bytes_callback (f(bytes) -> None): Will be called with extra
-                bytes inadvertantly read in get_cursor_position(). If not
+                bytes inadvertently read in get_cursor_position(). If not
                 provided, a ValueError will be raised when this occurs.
         """
-        BaseWindow.__init__(self, out_stream=out_stream, hide_cursor=hide_cursor)
+        super().__init__(out_stream=out_stream, hide_cursor=hide_cursor)
         if in_stream is None:
             in_stream = sys.__stdin__
         self.in_stream = in_stream
-        self._last_cursor_column = None  # type: Optional[int]
-        self._last_cursor_row = None  # type: Optional[int]
+        # whether we can use blessed to handle some operations
+        self._use_blessed = (
+            self.out_stream == sys.__stdout__ and self.in_stream == sys.__stdin__
+        )
+        self._last_cursor_column: Optional[int] = None
+        self._last_cursor_row: Optional[int] = None
         self.keep_last_line = keep_last_line
-        self.cbreak = Cbreak(self.in_stream)
+        self.cbreak = (
+            Cbreak(self.in_stream) if not self._use_blessed else self.t.cbreak()
+        )
         self.extra_bytes_callback = extra_bytes_callback
 
         # whether another SIGWINCH is queued up
@@ -300,61 +283,69 @@ class CursorAwareWindow(BaseWindow):
         # in the cursor query code of cursor diff
         self.in_get_cursor_diff = False
 
-    def __enter__(self):
-        # type: () -> CursorAwareWindow
+    def __enter__(self) -> "CursorAwareWindow":
         self.cbreak.__enter__()
         self.top_usable_row, _ = self.get_cursor_position()
         self._orig_top_usable_row = self.top_usable_row
         logger.debug("initial top_usable_row: %d" % self.top_usable_row)
-        return BaseWindow.__enter__(self)
+        return super().__enter__()
 
-    def __exit__(self, type, value, traceback):
-        # type: (Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]) -> None
+    def __exit__(
+        self,
+        type: Optional[Type[BaseException]] = None,
+        value: Optional[BaseException] = None,
+        traceback: Optional[TracebackType] = None,
+    ) -> None:
         if self.keep_last_line:
             # just moves cursor down if not on last line
-            self.write(SCROLL_DOWN)
+            self.write(self.t.move_down)
 
-        self.write(FIRST_COLUMN)
+        self.write(self.t.move_x(0))
         self.write(self.t.clear_eos)
         self.write(self.t.clear_eol)
         self.cbreak.__exit__(type, value, traceback)
-        BaseWindow.__exit__(self, type, value, traceback)
+        super().__exit__(type, value, traceback)
 
-    def get_cursor_position(self):
-        # type: () -> Tuple[int, int]
+    def get_cursor_position(self) -> Tuple[int, int]:
         """Returns the terminal (row, column) of the cursor
 
-        0-indexed, like blessings cursor positions"""
+        0-indexed, like blessed cursor positions"""
+
+        if self._use_blessed:
+            return self.t.get_location()
+
         # TODO would this be cleaner as a parameter?
         in_stream = self.in_stream
 
         query_cursor_position = "\x1b[6n"
         self.write(query_cursor_position)
 
-        def retrying_read():
-            # type: () -> str
+        def retrying_read() -> str:
             while True:
                 try:
                     c = in_stream.read(1)
                     if c == "":
                         raise ValueError(
-                            "Stream should be blocking - should't"
+                            "Stream should be blocking - shouldn't"
                             " return ''. Returned %r so far",
                             (resp,),
                         )
                     return c
-                except IOError:
-                    raise ValueError("cursor get pos response read interrupted")
-                    # find out if this ever really happens - if so, continue
+                except OSError:
+                    # apparently sometimes this happens: the only documented
+                    # case is Terminal on a Ubuntu 17.10 VM on osx 10.13.
+                    # see issue #732
+                    logger.info("stdin.read(1) that should never error just errored.")
+                    continue
 
         resp = ""
         while True:
             c = retrying_read()
             resp += c
             m = re.search(
-                "(?P<extra>.*)"
-                "(?P<CSI>\x1b\[|\x9b)"
-                "(?P<row>\\d+);(?P<column>\\d+)R",
+                r"(?P<extra>.*)"
+                r"(?P<CSI>\x1b\[|\x9b)"
+                r"(?P<row>\d+);(?P<column>\d+)R",
                 resp,
                 re.DOTALL,
             )
@@ -363,7 +354,7 @@ class CursorAwareWindow(BaseWindow):
                 col = int(m.groupdict()["column"])
                 extra = m.groupdict()["extra"]
                 if extra:
-                    if self.extra_bytes_callback:
+                    if self.extra_bytes_callback is not None:
                         self.extra_bytes_callback(
                             # TODO how do we know that this works?
                             extra.encode(cast(TextIO, in_stream).encoding)
@@ -380,8 +371,7 @@ class CursorAwareWindow(BaseWindow):
                         )
                 return (row - 1, col - 1)
 
-    def get_cursor_vertical_diff(self):
-        # type: () -> int
+    def get_cursor_vertical_diff(self) -> int:
         """Returns the how far down the cursor moved since last render.
 
         Note:
@@ -410,8 +400,7 @@ class CursorAwareWindow(BaseWindow):
             if not self.another_sigwinch:
                 return cursor_dy
 
-    def _get_cursor_vertical_diff_once(self):
-        # type: () -> int
+    def _get_cursor_vertical_diff_once(self) -> int:
         """Returns the how far down the cursor moved."""
         old_top_usable_row = self.top_usable_row
         row, col = self.get_cursor_position()
@@ -435,8 +424,11 @@ class CursorAwareWindow(BaseWindow):
         self._last_cursor_row = row
         return cursor_dy
 
-    def render_to_terminal(self, array, cursor_pos=(0, 0)):
-        # type: (Union[FSArray, List[FmtStr]], Tuple[int, int]) -> int
+    def render_to_terminal(
+        self,
+        array: Union[FSArray, Sequence[FmtStr]],
+        cursor_pos: Tuple[int, int] = (0, 0),
+    ) -> int:
         """Renders array to terminal, returns the number of lines scrolled offscreen
 
         Returns:
@@ -466,7 +458,7 @@ class CursorAwareWindow(BaseWindow):
         if height != self._last_rendered_height or width != self._last_rendered_width:
             self.on_terminal_size_change(height, width)
 
-        current_lines_by_row = {}  # type: Dict[int, Optional[FmtStr]]
+        current_lines_by_row: Dict[int, Optional[FmtStr]] = {}
         rows_for_use = list(range(self.top_usable_row, height))
 
         # rows which we have content for and don't require scrolling
@@ -501,9 +493,7 @@ class CursorAwareWindow(BaseWindow):
                 self.top_usable_row -= 1
             else:
                 offscreen_scrolls += 1
-            current_lines_by_row = dict(
-                (k - 1, v) for k, v in current_lines_by_row.items()
-            )
+            current_lines_by_row = {k - 1: v for k, v in current_lines_by_row.items()}
             logger.debug("new top_usable_row: %d" % self.top_usable_row)
             # since scrolling moves the cursor
             self.write(self.t.move(height - 1, 0))
@@ -523,8 +513,7 @@ class CursorAwareWindow(BaseWindow):
         return offscreen_scrolls
 
 
-def demo():
-    # type: () -> None
+def demo() -> None:
     handler = logging.FileHandler(filename="display.log")
     logging.getLogger(__name__).setLevel(logging.DEBUG)
     logging.getLogger(__name__).addHandler(handler)
@@ -533,15 +522,14 @@ def demo():
     with FullscreenWindow(sys.stdout) as w:
         with input.Input(sys.stdin) as input_generator:
             rows, columns = w.t.height, w.t.width
-            while True:
-                c = input_generator.next()
-                assert isinstance(c, Text)
+            for c in input_generator:
+                assert isinstance(c, str)
                 if c == "":
                     sys.exit()  # same as raise SystemExit()
                 elif c == "h":
-                    a = w.array_from_text(
+                    a: Union[List[FmtStr], FSArray] = w.array_from_text(
                         "a for small array"
-                    )  # type: Union[List[FmtStr], FSArray]
+                    )
                 elif c == "a":
                     a = [fmtstr(c * columns) for _ in range(rows)]
                 elif c == "s":
@@ -565,8 +553,7 @@ def demo():
                 w.render_to_terminal(a)
 
 
-def main():
-    # type: () -> None
+def main() -> None:
     handler = logging.FileHandler(filename="display.log")
     logging.getLogger(__name__).setLevel(logging.DEBUG)
     logging.getLogger(__name__).addHandler(handler)
@@ -574,7 +561,7 @@ def main():
     w = FullscreenWindow(sys.stdout)
     rows, columns = w.t.height, w.t.width
     with w:
-        a = [fmtstr(((".row%r." % (row,)) * rows)[:columns]) for row in range(rows)]
+        a = [fmtstr(((f".row{row!r}.") * rows)[:columns]) for row in range(rows)]
         w.render_to_terminal(a)
 
 
